// Atoms.cpp: implementation of the Atoms class.
//
//////////////////////////////////////////////////////////////////////

#include "Atoms.h"

#define NULL 0	// XXX Needed from afx.h??

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


// Constructor: sets up variables, allocates memory
Atoms::Atoms(int num, double width, double height, double depth)
{

	boxSize = new double[DIMENSION];
	
	boxSize[0]=width;
	
	if (DIMENSION>1) boxSize[1]=height;
	if (DIMENSION>2) boxSize[2]=depth;

	// Arrays should have fixed length: Jacob Schiotz uses double (*position)[DIMENSION] rather than double **position
	position = NULL;
	velocity = NULL;
	forces = NULL;
	radius = 0.5;
	
	SetNumber(num);	// Memory allocated here


}

// Destructor: de-allocates memory
Atoms::~Atoms()
{
	delete [] position[0];
	delete [] position;
	delete [] velocity[0];
	delete [] velocity;
	delete [] forces[0];
	delete [] forces;
	delete [] boxSize;
}

// Resets number of atoms
void Atoms::SetNumber(int num)
{
	if (position != NULL)
	{
		delete [] position[0];
		delete [] position;
	}
	if (velocity != NULL)
	{
		delete [] velocity[0];
		delete [] velocity;
	}
	if (forces != NULL)
	{
		delete [] forces[0];
		delete [] forces;
	}

	this->num = num;
	position = new double*[num];
	position[0] = new double[num*DIMENSION];
	velocity = new double*[num];
	velocity[0] = new double[num*DIMENSION];
	forces = new double*[num];
	forces[0] = new double[num*DIMENSION];

	for (int i=0; i<num; i++) 
	{
		if (i>0)
		{
			position[i] = position[i-1]+DIMENSION;
			velocity[i] = velocity[i-1]+DIMENSION;
			forces[i] = forces[i-1]+DIMENSION;
		}
	}

}

// Gives number of atoms closer than sqrt(r2max) from atom number i
int Atoms::NumNeighbors(double r2max, int i)
{

	// Order N^2 algorithm: slow!
	// Should use neighbor cells to speed up this computation!

	int numNeighbors = 0;

	for (int j=0; j<num; j++)
	{
		if (j != i) 
		{
			double r2 = 0;
			for (int coord=0; coord<DIMENSION; coord++)
			{
				double dx = position[i][coord] - position[j][coord];	// Distance along direction coord
				if (dx < -0.5*boxSize[coord]) dx += boxSize[coord];	// Periodic boundary conditions
				if (dx > 0.5*boxSize[coord]) dx -= boxSize[coord];
				r2 += dx*dx;
			}
			if (r2 < r2max)	numNeighbors++;
		}
	}
	
	return numNeighbors;

}

