// Atoms.h: interface for the Atoms class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ATOMS_H
#define ATOMS_H

#ifndef SWIG
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
#endif

#include <math.h>

#define DIMENSION 2

class Atoms  
{

public:		// Can be accessed by anyone
	// Atoms() is a Constructor: allocates memory, sets up variables
	Atoms(int num=10, double width=10.0, double height=10.0, double depth=10.0);	// Depth not used if DIMENSION<3
	virtual ~Atoms();	// Destructor: frees memory, shuts down gracefully
	int Number() const {return num;}	// Access protected variables; definition here so compiler substitutes in-line for speed
	double GetRadius() const {return radius;}	// Radius of atom: used for display, will be used for potential
	void SetNumber(int num);	// Non-trivial! Not in-lined: see cpp file for definition
	double X(int i, int coord) const {return position[i][coord];}
	double V(int i, int coord) const {return velocity[i][coord];}
	void SetX(int i, int coord, double x)	// Automatically enforces box size
	{
		position[i][coord]=x;
		if (x >= boxSize[coord])
		{
			position[i][coord] -= boxSize[coord];
			if (x >= boxSize[coord]) 
				position[i][coord] = fmod(x,boxSize[coord]);	// Division and mod are very slow: use only when atoms go crazy
		}
		if (x < 0)
		{
			position[i][coord] += boxSize[coord];
			if (x < 0)
				position[i][coord] = fmod(x,boxSize[coord])+boxSize[coord];	// Division and mod are slow: see above
		}
	}
	void SetV(int i, int coord, double v) {velocity[i][coord]=v;}
	int NumNeighbors(double distance2max, int atom_number);
	const double *BoxSize() const {return boxSize;}	// Returns array of sizes for periodic boundary conditions

protected:		// Not accesible to outside
	int num;
	double radius;
	double **position;	// Allocated in constructor to size position[num][DIMENSION]
						// XXX Should be fixed-dimension arrays for efficiency
	double **velocity;	// e.g., double (*position)[DIMENSION] instead of double **position
	double **forces;	// Pre-allocated storage for forces: filled and returned by Forces()
	double *boxSize;

};


#endif // !defined(ATOMS_H)
